/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "cviewer.h"
#include "cviewportwatcher.h"
#include "cminimap.h"
#include "info.h"

#include <QShortcut>
#include <QVBoxLayout>


CScaleBar::CScaleBar(QWidget* cntnt,QWidget *parent):QWidget(parent),
  SCALEDEFAULT(100),
  KEYSTEP(5),
  _scaleMin(20),
  _scaleMax(500),
  _scale(SCALEDEFAULT),
  scScaleUp   (new QShortcut(QKeySequence(tr("Ctrl++")),cntnt)),
  scScaleDown (new QShortcut(QKeySequence(tr("Ctrl+-")),cntnt)),
  scScaleReset(new QShortcut(QKeySequence(tr("Ctrl+0")),cntnt))
{
 ui.setupUi(this);
 ui.schemeBtn->setVisible(false);

 setWindowIcon(QIcon(Info::dataPath()+"/system/image/crosti.png"));
 setAttribute(Qt::WA_DeleteOnClose);

 _scaleMin=ui.scaleSB->minimum();
 _scaleMax=ui.scaleSB->maximum();
 scScaleReset->setAutoRepeat(false);

 ui.schemeBtn->setIcon(QIcon(Info::dataPath()+"/system/image/2.png"));
 ui.scaleResetBtn->setIcon(QIcon(Info::dataPath()+"/system/image/zoomreset.png"));

 connect(scScaleDown,SIGNAL(activated()),SLOT(scaleDown()));
 connect(scScaleUp,SIGNAL(activated()),SLOT(scaleUp()));
 connect(scScaleReset,SIGNAL(activated()),SLOT(scaleResetBtnPress()));

 connect(ui.scaleSB,SIGNAL(valueChanged(int)),SLOT(setScale(int)));
 connect(ui.scaleResetBtn,SIGNAL(clicked()),SLOT(scaleResetBtnPress()));
}


CScaleBar::~CScaleBar()
{
  delete scScaleReset;
  delete scScaleDown;
  delete scScaleUp;
}


void CScaleBar::setScale(int scl)
{
  _scale=scl;
  if(_scale>_scaleMax)_scale=_scaleMax;
  if(_scale<_scaleMin)_scale=_scaleMin;
  emit scaleChanged(qreal(_scale)/SCALEDEFAULT);
}


void CScaleBar::scaleChange(int dscale)
{
  setScale(_scale+dscale);
  setScaleSB(_scale);
}


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------


CViewer::CViewer(QWidget *parent):QWidget(parent),
  layout(new QVBoxLayout),
  scaleBar(new CScaleBar(this)),
  MOUSESTEP(10)
{
  layout->setSpacing(0);
  layout->setMargin(0);
  layout->addWidget(scaleBar);
  setAttribute(Qt::WA_DeleteOnClose);
  setLayout(layout);
}


CViewer::~CViewer()
{
  delete scaleBar;
  delete layout;
}


void CViewer::wheelEvent(QWheelEvent *e)
{
  if(e->modifiers().testFlag(Qt::ControlModifier))
    scaleBar->scaleChange((e->delta()>0) ? MOUSESTEP : -MOUSESTEP);
}


void CViewer::updateMiniMap(CMiniMap *miniMap)
{
    connect(viewPortWatcher(), SIGNAL(visibleRectChanged(const QRectF&)), miniMap, SLOT(setViewPortRect(const QRectF&)));
    connect(miniMap, SIGNAL(viewPortPositionChanged(const QPointF&)), viewPortWatcher(), SLOT(viewPortPositionChanged(const QPointF&)));
    miniMap->setImage(image());
    viewPortWatcher()->viewPortChanged();
}


void CViewer::disconnectViewPortWatcher()
{
    viewPortWatcher()->disconnect(SIGNAL(visibleRectChanged(const QRectF&)));
}
